# Metaverse Setup Module 
# 
# This module contains all the functions needed for the setup 
# Window to work.
#
# Module Name		- Setup Module
# Current Revision No.	- 0.1
# Current Maintainter 	- Cruise <cruise@drunkenbastards.org>
# Last update 		- Sun Sep  5 21:24:33 EDT 1999
# Sourced By		- InitMainWindow
#
# Copyright (C) 1999 David Gale <cruise@drunkenbastards.org>
# For more information visit http://metaverse.daytona.com/Metaverse/
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
# USA.

#
# This sets up the setup window.
#
proc Setup {} {
	global MV
	
	if [winfo exists .setup] {destroy .setup}
	toplevel .setup
	wm title .setup "Metaverse Setup"
	wm resizable .setup 0 0
	frame .setup.na
	frame .setup.na.l
	frame .setup.na.l.t
	frame .setup.na.l.b
	frame .setup.na.l.t.nick
	frame .setup.na.l.b.avatars
	frame .setup.na.r
	frame .setup.cmds

	scale .setup.na.r.bspeed -from 1 -to 20 -variable MV(staytime) \
		-orient horizontal -label "Baloon Speed" -relief raised \
		-borderwidth 2
	scale .setup.na.r.mspeed -from 1 -to 20 -variable MV(movespeed) \
		-orient horizontal -label "Move Speed" -relief raised \
		-borderwidth 2


	label .setup.na.l.t.nick.txt -text " Nick Name "
	entry .setup.na.l.t.nick.entry
	menubutton .setup.na.l.b.avatars.menu -text "Avatar" -menu \
		.setup.na.l.b.avatars.menu.m -relief raised -borderwidth 2
	menu .setup.na.l.b.avatars.menu.m


	set anims 0
	set depth 0
	set count 0
	foreach file [lsort -ascii [glob -nocomplain "$MV(anims)/*"]] {
		if !$depth {
			incr depth
			.setup.na.l.b.avatars.menu.m add cascade -menu \
				.setup.na.l.b.avatars.menu.m.m$depth -label "Avatars $depth"
			menu .setup.na.l.b.avatars.menu.m.m$depth
		}
		set MV(avatar_setup.$count) "[file tail $file]"
		.setup.na.l.b.avatars.menu.m.m$depth add command -label [file tail $file] \
			-command "AnimateMe $count 2"
		incr anims
		incr count
		if {$anims > 30} {
			incr depth
			.setup.na.l.b.avatars.menu.m add cascade -menu \
				.setup.na.l.b.avatars.menu.m.m$depth -label "Avatars $depth"
			menu .setup.na.l.b.avatars.menu.m.m$depth
			set anims 0
		}
	}
	entry .setup.na.l.b.avatars.entry

	button .setup.cmds.cancel -text "Cancel" -command "destroy .setup"
	button .setup.cmds.save -text "Save & Apply" -command "SaveSetup 1"
	button .setup.cmds.apply -text "Apply Only" -command "SaveSetup 2"
	button .setup.cmds.saveo -text "Save Only" -command "SaveSetup 3"

	pack .setup.na -side top -fill both -expand y
	pack .setup.na.l .setup.na.r -side left -fill both -expand y
	pack .setup.na.l.t -side top -fill both -expand y
	pack .setup.na.l.b -side bottom -fill both -expand y
	pack .setup.na.l.t.nick -fill both -expand y
	pack .setup.na.l.b.avatars -fill both -expand y
	pack .setup.na.l.t.nick.txt -side left
	pack .setup.na.l.t.nick.entry -side left -fill both -expand y
	pack .setup.na.l.b.avatars.menu -side left
	pack .setup.na.l.b.avatars.entry -side left -fill both -expand y
	pack .setup.cmds -fill both -expand y
	pack .setup.cmds.cancel .setup.cmds.save .setup.cmds.apply .setup.cmds.saveo -side left -fill both \
		-expand y
	pack .setup.na.r.bspeed .setup.na.r.mspeed -side left -fill both \
		-expand y

	.setup.na.l.t.nick.entry insert end $MV(nick)
	.setup.na.l.b.avatars.entry insert end $MV(anim)
	
	bind .setup.na.l.t.nick.entry <KeyRelease> {CheckNick}
	bind .setup.na.l.t.nick.entry <ButtonPress> {CheckNick}
}

proc SaveSetup {how} {
	global MV

	set outfile [open "$MV(configfile)" w]
	puts $outfile "set MV(nick) \"[.setup.na.l.t.nick.entry get]\""
	puts $outfile "set MV(anim) \"[.setup.na.l.b.avatars.entry get]\""
	puts $outfile "set MV(staytime) $MV(staytime)"
	puts $outfile "set MV(movespeed) $MV(movespeed)"
	close $outfile
	set MV($MV(nick).haschat) 0
	set MV($MV(nick).chatque) {}
	if $MV(names) {
		KillName $MV(nick)
	}
	set newnick [.setup.na.l.t.nick.entry get]
	if {$MV(nick) != $newnick} {
		set MV($newnick.haschat) 0
		set MV($newnick.chatque) {}
		ChangeNick "$newnick"
	}
	set MV(anim) [.setup.na.l.b.avatars.entry get]
	AnimateMe "[.setup.na.l.b.avatars.entry get]" 3
	SendToServer "NICK $MV(nick)"
	destroy .setup
}
