# Metaverse URL Display Module
# 
# Contains functions which are related to displaying
# and clicking on URLS offered by people.
#
# Module Name		- URL Module
# Current Revision No.	- 0.3
# Current Maintainter 	- Cruise <cruise@drunkenbastards.org>
# Last update 		- Sun Sep  5 21:24:33 EDT 1999
# Sourced By		- InitMain.tcl
#
# Copyright (C) 1999 David Gale <cruise@drunkenbastards.org>
# For more information visit http://metaverse.daytona.com/Metaverse/
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
# USA.

# URL's use icons to identify them. We pre-create the photos of them at
# startup.

image create photo URL_UNKNOWN -file "$MV(icondir)/unknown.gif"
image create photo URL_DCCFILE -file "$MV(icondir)/dccfile.gif"
image create photo URL_METAVERSE -file "$MV(icondir)/mv2.gif"

proc ProcURL {who what} {
	global MV

	# Get a random screen position (allowing 50sq around it.)
	# This gives us a grid 16x12.

	set foundspot 0
	while {$foundspot == 0} {
		set x [expr int(rand() * 15)]
		set y [expr int(rand() * 11)]
		if !$MV(URLS.$x.$y) {
			set MV(URLS.$x.$y) 1
			set foundspot 1
		}
		update idletasks
	}
	if $MV(debug) {puts "(URL) Using $x $y"}

	set display 0
	set type [URLType $what]
	switch -exact -- $type {
		"FILE" {
			set display 1
			set img "URL_DCCFILE"
		}
		"METAVERSE" {
			set display 1
			set img "URL_METAVERSE"
		}
		default {
			set display 1
			set img "URL_UNKNOWN"
		}
	}

	if $display {
		set MV(urlicon.$x.$y) [.top.c create image [lindex $MV(URLS.x) $x] [lindex $MV(URLS.y) $y] -image $img]
		after [expr $MV(urltimeout) * 1000] "DeleteUrl $x $y"
		.top.c bind $MV(urlicon.$x.$y) <Button> "UrlClick $x $y"
		set MV(url.$x.$y.from) $who
		set MV(url.$x.$y.type) $type
		set MV(url.$x.$y.what) $what
	}
}

proc UrlClick {x y} {
	global MV

	switch -exact -- $MV(url.$x.$y.type) {
		"FILE" {
			set Type "File Offer"
			set cmd "URL_DCCGet"
		}
		"EMAIL" {
			set Type "Email Address"
			set cmd "URL_KillOpts"
		}
		"WEB" {
			set Type "World Wide Web Address"
			set cmd "URL_KillOpts"
		}
		"METAVERSE" {
			set Type "Metaverse Server Link"
			set cmd "URL_Metaverse"
		}
		default {
			set Type "Unknown URL Type"
			set cmd "URL_KillOpts"
		}
	}

	set where n
	set dir left
	if {$x < 7 && $y < 5} {set where nw;set dir left}
	if {$x < 7 && $y > 5} {set where sw;set dir left}
	if {$x < 7 && $y == 5} {set where w;set dir left}
	if {$x > 7 && $y == 5} {set where e;set dir right}
	if {$x > 7 && $y < 5} {set where ne;set dir right}
	if {$x > 7 && $y > 5} {set where se;set dir right}
	if {$x == 7 && $y > 5} {set where s;set dir left}
	if {$x == 7 && $y < 5} {set where n;set dir left}
	if {$x == 7 && $y == 5} {set where center;set dir left}

	if [winfo exists .top.c.dlopts] {destroy .top.c.dlopts}
	frame .top.c.dlopts -relief raised -borderwidth 2 \
		-bg $MV(colors.url.frames.bg)

	set MV(urlopt.$x.$y) [.top.c create window [lindex $MV(URLS.x) $x] [lindex $MV(URLS.y) $y] -anchor $where -window .top.c.dlopts]

	frame .top.c.dlopts.info1 -relief sunken -borderwidth 2 \
		-bg $MV(colors.url.frames.bg)
	frame .top.c.dlopts.info2 -relief sunken -borderwidth 2 \
		-bg $MV(colors.url.frames.bg)
	frame .top.c.dlopts.info3 -relief sunken -borderwidth 2 \
		-bg $MV(colors.url.frames.bg)

	label .top.c.dlopts.info1.txt -relief raised -borderwidth 2 \
		-text "From" -width 15 \
		-fg $MV(colors.url.labels.fg) \
		-bg $MV(colors.url.labels.bg)
	label .top.c.dlopts.info2.txt -relief raised -borderwidth 2 \
		-text "URL" -width 15 \
		-fg $MV(colors.url.labels.fg) \
		-bg $MV(colors.url.labels.bg)
	label .top.c.dlopts.info3.txt -relief raised -borderwidth 2 \
		-text "Type" -width 15 \
		-fg $MV(colors.url.labels.fg) \
		-bg $MV(colors.url.labels.bg)
	label .top.c.dlopts.info1.info -relief sunken -borderwidth 2 \
		-text "$MV(url.$x.$y.from)" \
		-fg $MV(colors.url.entries.fg) \
		-bg $MV(colors.url.entries.bg)
	label .top.c.dlopts.info2.info -relief sunken -borderwidth 2 \
		-text "$MV(url.$x.$y.what)" \
		-fg $MV(colors.url.entries.fg) \
		-bg $MV(colors.url.entries.bg)
	label .top.c.dlopts.info3.info -relief sunken -borderwidth 2 \
		-text "$Type" \
		-fg $MV(colors.url.entries.fg) \
		-bg $MV(colors.url.entries.bg)

	button .top.c.dlopts.get -text "Open/Save" \
		-command "$cmd $x $y" \
		-fg $MV(colors.url.buttons.fg) \
		-bg $MV(colors.url.buttons.bg) \
		-activeforeground $MV(colors.url.buttons.afg) \
		-activebackground $MV(colors.url.buttons.abg)
	button .top.c.dlopts.ign -text "Ignore" \
		-command "URL_KillOpts $x $y" \
		-fg $MV(colors.url.buttons.fg) \
		-bg $MV(colors.url.buttons.bg) \
		-activeforeground $MV(colors.url.buttons.afg) \
		-activebackground $MV(colors.url.buttons.abg)
	button .top.c.dlopts.ref -text "Refuse" \
		-command "URL_KillOpts $x $y" \
		-fg $MV(colors.url.buttons.fg) \
		-bg $MV(colors.url.buttons.bg) \
		-activeforeground $MV(colors.url.buttons.afg) \
		-activebackground $MV(colors.url.buttons.abg)

	pack .top.c.dlopts.info1 -fill both -expand y
	pack .top.c.dlopts.info2 -fill both -expand y
	pack .top.c.dlopts.info3 -fill both -expand y
	pack .top.c.dlopts.info1.txt -side left -fill y
	pack .top.c.dlopts.info2.txt -side left -fill y
	pack .top.c.dlopts.info3.txt -side left -fill y
	pack .top.c.dlopts.info1.info -side left -expand y -fill both
	pack .top.c.dlopts.info2.info -side left -expand y -fill both
	pack .top.c.dlopts.info3.info -side left -expand y -fill both

	pack .top.c.dlopts.get .top.c.dlopts.ign .top.c.dlopts.ref -side $dir

}

proc DeleteUrl {x y} {
	global MV

	catch {.top.c delete $MV(urlicon.$x.$y)}
	set MV(URLS.$x.$y) 0
}

proc URLType {url} {
	global MV

	set parms [split $url ":"]
	switch -exact -- [string tolower [lindex $parms 0]] {
		"file" {set type "FILE"}
		"http" {set type "WEB"}
		"mailto" {set type "EMAIL"}
		"metaverse" {set type "METAVERSE"}
		default {set type "UNKNOWN"}
	}
	return $type
}

proc URL_DCCGet {x y} {
	global MV

	destroy .top.c.dlopts
	set parms [split $MV(url.$x.$y.what) "/"]
	set hp [split [lindex $parms 2] ":"]
	set fs [split [lindex $parms 3] ":"]
	DeleteUrl $x $y
	DCCGet [lindex $hp 0] [lindex $hp 1] [lindex $fs 0] [lindex $fs 1] DOWNLOAD $MV(url.$x.$y.from)
}

proc URL_Metaverse {x y} {
	global MV

	destroy .top.c.dlopts
	set parms [split $MV(url.$x.$y.what) "/"]
	set hp [split [lindex $parms 2] ":"]
	DeleteUrl $x $y
	Disconnect
	set MV(roomhost) [lindex $hp 0]
	set MV(roomport) [lindex $hp 1] 
	Connect
}

proc URL_KillOpts {x y} {
	destroy .top.c.dlopts
}
